<?php

namespace App\Http\Middleware;

use App\Model\SystemRecord;
use Closure;
use Illuminate\Support\Facades\Log;

class RequestLog
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        if (getenv('HTTP_CLIENT_IP')) {
            $ip = getenv('HTTP_CLIENT_IP');
        }
        elseif (getenv('HTTP_X_REAL_IP')) {
            $ip = getenv('HTTP_X_REAL_IP');
        } elseif (getenv('HTTP_X_FORWARDED_FOR')) {
            $ip = getenv('HTTP_X_FORWARDED_FOR');
            $ips = explode(',', $ip);
            $ip = $ips[0];
        } elseif (getenv('REMOTE_ADDR')) {
            $ip = getenv('REMOTE_ADDR');
        } else {
            $ip = '0.0.0.0';
        }
        $params = $request->all();
        $uri = $request->path();
        Log::channel('request')->info('request', ['param'=>json_encode($params),'ip' => $ip,'uri'=>$uri]);

        $map = config("routemap.map");
        if($uri == 'api/login'){
            $content = $map[$uri];
            $account = '';
            if(! empty($params['account'])){
                $account = $params['account'];
            }
            SystemRecord::insert([
                'account' => $account,
                'crt_time' => time(),
                'ip' => $ip,
                'content' => $content
            ]);
        }

        return $next($request);
    }
}
